@@ -27,7 +27,7 @@
 */
 
 #include <config.h>
-
+#include "eel-i18n.h"
 #include "eel-vfs-extensions.h"
 #include "eel-lib-self-check-functions.h"
 
@@ -652,6 +652,50 @@ eel_uri_is_in_trash (const char *uri)
 	return result;
 }
 
+gboolean
+eel_uri_is_desktop (const char *uri)
+{
+	return eel_istr_has_prefix (uri, EEL_DESKTOP_URI);
+}
+
+char *
+eel_make_valid_utf8 (const char *name)
+{
+	GString *string;
+	const char *remainder, *invalid;
+	int remaining_bytes, valid_bytes;
+
+	string = NULL;
+	remainder = name;
+	remaining_bytes = strlen (name);
+
+	while (remaining_bytes != 0) {
+		if (g_utf8_validate (remainder, remaining_bytes, &invalid)) {
+			break;
+		}
+		valid_bytes = invalid - remainder;
+
+		if (string == NULL) {
+			string = g_string_sized_new (remaining_bytes);
+		}
+		g_string_append_len (string, remainder, valid_bytes);
+		g_string_append_c (string, '?');
+
+		remaining_bytes -= valid_bytes + 1;
+		remainder = invalid + 1;
+	}
+
+	if (string == NULL) {
+		return g_strdup (name);
+	}
+
+	g_string_append (string, remainder);
+	g_string_append (string, _(" (invalid Unicode)"));
+	g_assert (g_utf8_validate (string->str, -1, NULL));
+
+	return g_string_free (string, FALSE);
+}
+
 static char *
 eel_format_uri_for_display_internal (const char *uri, gboolean filenames_are_locale_encoded)
 {
@@ -678,8 +722,11 @@ eel_format_uri_for_display_internal (const char *uri, gboolean filenames_are_loc
 		}
 	}
 
-	g_assert (!canonical_uri ||
-		  g_utf8_validate (canonical_uri, -1, NULL));
+	if (canonical_uri && !g_utf8_validate (canonical_uri, -1, NULL)) {
+		utf8_path = eel_make_valid_utf8 (canonical_uri);
+		g_free (canonical_uri);
+		canonical_uri = utf8_path;
+	}
 
 	g_free (path);
 	return canonical_uri;
@@ -1592,6 +1639,85 @@ eel_copy_uri_simple ( const char *source_uri, const char *dest_uri)
 	return  result;
 }
 
+
+gboolean
+eel_vfs_has_capability_uri (GnomeVFSURI     *uri,
+			    EelVfsCapability capability)
+{
+	gboolean ret;
+	const char *scheme;
+
+	g_return_val_if_fail (uri != NULL, FALSE);
+
+	scheme = gnome_vfs_uri_get_scheme (uri);
+
+	switch (capability) {
+	case EEL_VFS_CAPABILITY_SAFE_TO_EXECUTE:
+		ret = gnome_vfs_uri_is_local (uri) ||
+			!strcmp (scheme, "file");
+		break;
+	case EEL_VFS_CAPABILITY_IS_REMOTE_AND_SLOW:
+		ret = !(gnome_vfs_uri_is_local (uri) ||
+			!strcmp (scheme, "file"));
+		break;
+	default:
+		ret = FALSE;
+		g_assert_not_reached ();
+	}
+
+	return ret;
+}
+
+gboolean
+eel_vfs_has_capability (const char       *text_uri,
+			EelVfsCapability  capability)
+{
+	gboolean ret;
+	GnomeVFSURI *uri;
+	
+	g_return_val_if_fail (text_uri != NULL, FALSE);
+
+	uri = gnome_vfs_uri_new (text_uri);
+
+	ret = eel_vfs_has_capability_uri (uri, capability);
+	
+	gnome_vfs_uri_unref (uri);
+
+	return ret;
+}
+
+void
+eel_filename_get_rename_region (const char           *filename,
+				int                  *start_offset,
+				int                  *end_offset)
+{
+	const char *end, *end2;
+
+	*start_offset = 0;
+	
+	end = strrchr (filename, '.');
+
+	if (end && end != filename) {
+		if (strcmp (end, ".gz") == 0 ||
+		    strcmp (end, ".bz2") == 0 ||
+		    strcmp (end, ".sit") == 0 ||
+		    strcmp (end, ".Z") == 0) {
+			end2 = end - 1;
+			while (end2 > filename &&
+			       *end2 != '.') {
+				end2--;
+			}
+			if (end2 != filename) {
+				end = end2;
+			}
+		}
+		*end_offset = g_utf8_pointer_to_offset (filename, end);
+	} else {
+		*end_offset = g_utf8_strlen (filename, -1);
+	}
+}
+
+
 #if !defined (EEL_OMIT_SELF_CHECK)
 
 void
@@ -1779,9 +1905,9 @@ eel_self_check_vfs_extensions (void)
 	 */
 	EEL_CHECK_STRING_RESULT (eel_make_uri_canonical ("file:trash"), "file:trash");
 	EEL_CHECK_STRING_RESULT (eel_make_uri_canonical ("//trash"), "file:///trash");
-	EEL_CHECK_STRING_RESULT (eel_make_uri_canonical ("file:"), "file:");
+	EEL_CHECK_STRING_RESULT (eel_make_uri_canonical ("file:"), "file:///");
 	EEL_CHECK_STRING_RESULT (eel_make_uri_canonical ("trash"), "file:///trash");
-	EEL_CHECK_STRING_RESULT (eel_make_uri_canonical ("glorp:"), "glorp:");
+	EEL_CHECK_STRING_RESULT (eel_make_uri_canonical ("glorp:"), "glorp:///");
 	EEL_CHECK_STRING_RESULT (eel_make_uri_canonical ("TRASH:XXX"), EEL_TRASH_URI);
 	EEL_CHECK_STRING_RESULT (eel_make_uri_canonical ("trash:xxx"), EEL_TRASH_URI);
 	EEL_CHECK_STRING_RESULT (eel_make_uri_canonical ("GNOME-TRASH:XXX"), EEL_TRASH_URI);
@@ -1837,7 +1963,7 @@ eel_self_check_vfs_extensions (void)
 #undef TEST_PARTIAL
 
 	EEL_CHECK_STRING_RESULT (eel_format_uri_for_display (""), "/");
-	EEL_CHECK_STRING_RESULT (eel_format_uri_for_display (":"), ":");
+	EEL_CHECK_STRING_RESULT (eel_format_uri_for_display (":"), ":///");
 	EEL_CHECK_STRING_RESULT (eel_format_uri_for_display ("file:///h/user"), "/h/user");
 	EEL_CHECK_STRING_RESULT (eel_format_uri_for_display ("file:///%68/user/foo%2ehtml"), "/h/user/foo.html");
 	EEL_CHECK_STRING_RESULT (eel_format_uri_for_display ("file:///h/user/foo.html#fragment"), "file:///h/user/foo.html#fragment");
